import math

def calc_jacob(mat,n,eps,cmax):
  '''
　　ヤコビ法で計算

　　入力引数：計算マトリクス（実対象行列）、次数、収束値、計算回数

　　戻り値：固有値・マトリクス、固有ベクトル・マトリクス、計算回数、収束値

　　参照：FORTRAN数値計算とプログラミング
       国井利泰監修　中村明子、伊藤文子著　1971 共立出版
   　  1.5　固有値　A.ヤコビ法　EX-1-5-A　p-91 ~ 100
　　　　より、Fortranの手順をPythonに修正したもの。
  '''
  u = [[ 0.0 for i in range(n)] for j in range(n)]
  for i in range(n):
    u[i][i] = 1.0
  n1 = n-1
  kout = 0
  while True:
    amax = 0.0
    for i in range(n1):
      i1 = i+1
      for j in range(i1,n):
        w = math.fabs(mat[i][j])
        if amax < w:
          amax = w
          k=i
          l=j

    if eps >= amax:
      break
    if kout >= cmax:
      break

    kout = kout + 1
    a1 = mat[k][k]
    a2 = mat[l][l]
    a3 = mat[k][l]
    w = (a1-a2) * 0.5
    if w < 0.0:
      af = a3
    else:
      af = -1.0 * a3

    if w == 0.0:
      z = math.sqrt(4.0*math.pow(a3,2))
      t = 2.0*af / z
    else:
      w = af/w
      t = w/(math.sqrt(w*w+1.0)+1.0)

    zz = t*t+1.0
    c = math.sqrt(1.0/zz)
    s = t * c
    for i in range(n):
      aw = u[i][k]
      az = u[i][l]
      u[i][k] = aw*c + az*s
      u[i][l] = -aw*s + az*c
      if i ==k:
        continue
      if i == l:
        continue
      aw = mat[i][k]
      az = mat[i][l]
      mat[i][k] = aw*c + az*s
      mat[i][l] = -aw*s + az*c
      mat[k][i] = mat[i][k]
      mat[l][i] = mat[i][l]

    asc2 = 2.0 * a3 * s * c
    mat[k][k] = a1 * math.pow(c,2) + a2 * math.pow(s,2) + asc2
    mat[l][l] = a1 * math.pow(s,2) + a2 * math.pow(c,2) - asc2
    mat[k][l] = 0.0
    mat[l][k] = 0.0
  return [mat, u, kout, amax]
