import math

def jac(a,n,kmax):
  '''
   ヤコビ法による固有値計算   
   (科学技術計算ハンドブック　基礎編C版　戸川隼人著　サイエンス社)

       a : matrix
       n : 次数
     kmax: 計算回数制限値

  '''
  # 収束判定定数の算定
  gmax = 0.0
  for i in range(n):
    s = 0.0
    for j in range(n):
      s += math.fabs(a[i][j])
    if s > gmax:
      gmax = s

  eps = 2.2204460492503131E-016*gmax
  # 固有ベクトルの計算の準備
  v = []
  for i in range(n):
    v.append([0.0 for j in range(n)])
    v[i][i] = 1.0
  
  # メイン･ループ
  for kaisuu in range(1, kmax+1):
    # 収束判定
    apqmax = 0.0
    for p in range(n-1):
      for q in range(p+1, n):
        apq = math.fabs(a[p][q]);
        if apq > apqmax:
          apqmax=apq
    if apqmax < eps:
      break
    # 走査
    for p in range(n-1):
      for q in range(p+1, n):
        apq = a[p][q]
        if math.fabs(apq) < eps:
          break
        # ｓとｃの算定
        wa = (a[p][p]+a[q][q]) * 0.5
        sa = (a[p][p]-a[q][q]) * 0.5
        bunbo = math.sqrt(sa*sa+a[p][q]**2)
        if sa > 0.0:
          c = math.sqrt(1.0+sa/bunbo)/math.sqrt(2)
          s = apq / (2.0*c*bunbo)
          # 対角要素
          a[p][p] = wa + bunbo
          a[q][q] = wa - bunbo
        else:
          c = math.sqrt(1.0-sa/bunbo)/math.sqrt(2)
          s = (-apq)/(2.0*c*bunbo)
          # 対角要素
          a[p][p] = wa - bunbo
          a[q][q] = wa + bunbo
        # 非対角要素
        a[p][q] = a[q][p] = 0.0
        for j in range(p):
          apj = a[p][j]
          aqj = a[q][j]
          a[j][p] = a[p][j]= apj*c+aqj*s
          a[j][q] = a[q][j]= (-apj*s+aqj*c)
        for j in range(p+1,q):
          apj = a[p][j]
          aqj = a[q][j]
          a[j][p] = a[p][j] =  apj*c+aqj*s
          a[j][q] = a[q][j] = (-apj*s+aqj*c)
        for j in range(q+1,n):
          apj = a[p][j]
          aqj = a[q][j]
          a[j][p] = a[p][j] =  apj*c+aqj*s
          a[j][q] = a[q][j] = (-apj*s+aqj*c)
        for i in range(n):
          vip = v[i][p]
          viq = v[i][q]
          v[i][p] = vip*c+viq*s
          v[i][q] = (-vip*s+viq*c)
    eps = eps * 1.05;
  if kaisuu > kmax:
    print("収束しませんでした")
  return a, v, kaisuu
      
