import numpy as np
import random as r

from sci_calc_jacob import jac as eig_jac
from fp_jacob import calc_jacob as fp_jacob
from jacobi_eigenvalue import jacobi_eigenvalue as jburkardt_jacob

N = 4
RMAX = 999

def gen_matrix(n):
  '''
    対称行列生成　ｎ　ｘ　ｎ
  '''
  m = np.array([[]])
  for i in range(n):
    if len(m[0]) == 0:
      m = np.array([[r.randrange(-RMAX,RMAX)/10.0 for j in range(n)]])
    else:
      m = np.append(m, [[r.randrange(-RMAX,RMAX)/10.0 for j in range(n)]], axis=0)
  m = (m+m.T)/2
  return m

def eig_check(A, value, vector):
    r = np.zeros(value.shape[0])
    i = 0
    for l, v in zip(value, vector.T):
        r[i] = np.linalg.norm(A.dot(v) - l*v)
        i += 1
    return r

def result(str, n, A, evalue, evect):
  print("\n■■■■■■■■■　計算結果 No.%d　■■■■(%s)■■■■■" % (n,str))
  n = len(A)
  print("固有値\n",[evalue[i][i] for i in range(N)])
  print("固有ベクタ\n",evect)
  print("　　■■■　検証１　■■■")
  print(eig_check(A, np.array([evalue[i][i] for i in range(n)]), evect))
  print("　　■■■　検証２　■■■")
  print(np.dot(A,evect)-np.dot(evect,evalue))

def chk_symmetric(mat, n):
  for i in range(n):
    for j in range(n):
      if mat[i][j] != mat[j][i]:
        return 0
  return 1

import warnings
warnings.filterwarnings('ignore')

meps = 2.2204460492503131E-016

for tp in range(2):
  #a = np.array([[1.0, -1, -1], [-1, 1, -1], [-1, -1, 1]])
  a = gen_matrix(N)
  print("◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎")
  print("◎◎◎◎◎◎◎◎◎◎◎計算開始◎◎◎◎◎◎◎◎◎◎")
  print("◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎◎")
  print("No.%d 計算行列 %d x %d" % (tp+1,N,N))
  print(a)
  if not chk_symmetric(a, N):
    print("対称行列ではありません")
  np_inmatrix = np.array([e.copy() for e in a]) # copy matrix
  reig = eig_jac(a,N, 100) # 科学技術計算ハンドブック　サイエンス社
  for i in range(N):
    for j in range(N):
      if i != j:
        reig[0][i][j] = 0.0
  np_e = np.array(reig[0]) # eig value
  np_v = np.array(reig[1]) # eig vector
  result("科学技術計算ハンドブック",tp+1, np_inmatrix, np_e, np_v)
  a = np_inmatrix.tolist()
  reig = fp_jacob(a,N,meps,30) # FORTRAN数値計算とプログラミング 　共立出版
  for i in range(N):
    for j in range(N):
      if i != j:
        reig[0][i][j] = 0.0
  np_e = np.array(reig[0]) # eig value
  np_v = np.array(reig[1]) # eig vector
  result("FORTRAN数値計算とプログラミング",tp+1, np_inmatrix, np_e, np_v)

  a = np.array([e.copy() for e in np_inmatrix]) # copy matrix
  reig = jburkardt_jacob( N, a, 100 )  # sc.fsu.edu/~jburkardt/py_src/jacobi_eigenvalue
  np_e = np.array([[0.0 for j in range(N)] for j in range(N)])
  for i in range(N):          # eig value
    np_e[i][i] = reig[1][i]
  np_v = np.array(reig[0])    # eig vector
  result("sc.fsu.edu/ j.burkardt",tp+1, np_inmatrix, np_e, np_v)
  a = np_inmatrix.tolist()
  np_e,np_v = np.linalg.eig(a)

  np_value = np.array([[0.0 for j in range(N)] for j in range(N)])
  for i in range(N):
    np_value[i][i] = np_e[i]  # eig value
  result("PYTHON NUMPY linalg.eig",tp+1, np_inmatrix, np_value, np_v)
